/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




package org.greenstone.gatherer.cdm;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Copyright:    Copyright (c) 2001
 * Company:      The University of Waikato
 * Written:      08/05/02
 * Revised:      17/11/02 - Commented
 **************************************************************************************/
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.Language;
/** This class manages the language commands, remembering both a list of languages to build indexes in, plus the default language.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class LanguageManager
    extends DefaultListModel {
    /** A reference to the collection design manager. */
    private CollectionDesignManager manager = null;
    /** The visual controls for this manager. */
    private Control controls = null;
    /** A reference to the Gatherer. */
    private Gatherer gatherer = null;
    /** A reference to this class as a model, for the inner controls class. */
    private ListModel model = null;
    /** A hashtable of code->name mappings of known languages. */
    private LinkedHashMap known_languages = null;
    /** The default language object. */
    private Language default_language = null;

    static final private Dimension LABEL_SIZE = new Dimension(125,25);
    /** Constructor.
     * @param gatherer A reference to the <strong>Gatherer</strong>.
     * @param manager A reference to the <strong>CollectionDesignManager</strong>.
     */
    public LanguageManager(Gatherer gatherer, CollectionDesignManager manager) {
	super();
	this.gatherer = gatherer;
	this.known_languages = new LinkedHashMap();
	this.manager = manager;
	this.model = this;
	loadLanguages();
    }
    /** Method to add a new language.
	  * @param language The <strong>Language</strong> to add.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  */
    public void addLanguage(Language language) {
	if(!contains(language)) {
				// Add alphabetically.
	    for(int index = 0; index < size(); index++) {
		Language sibling = (Language) get(index);
		int position = language.compareTo(sibling);
		// Sibling is before language.
		if(position > 0) {
		    // Carry on.
		}
		// Language is equal to, or before sibling. Insert it.
		else if(position == 0 || position < 0) {
		    add(index, language);
		    gatherer.c_man.configurationChanged();
		    return;
		}
	    }
				// If we got this far, we haven't inserted language, and we are out of model so.
	    addElement(language);
	    gatherer.c_man.configurationChanged();
	}
    }
    /** Method to retrieve the control for this manager.
	  * @return A <strong>JPanel</strong> containing the controls.
	  */
    public JPanel getControls() {
	if(controls == null) {
	    controls = new Control();
	}
	return controls;
    }
    /** Method to retrieve the default language code.
	  * @return A <strong>Language</strong> containing a two letter code.
	  */
    public Language getDefaultLanguage() {
	// If no default is set...
	if(default_language == null) {
				// And we have other assigned languages, use the first one of them.
	    if(size() >= 1) {
		default_language = (Language) get(0);
	    }
				// And we have nothing else, use English.
	    else {
		default_language = getLanguage("en", false);
		// Remember to add it.
		addLanguage(default_language);
	    }
	}
	return default_language;
    }
    /** Method to retrieve a certain language object by its code.
     * @param code The two letter code of a language, as a <strong>String</strong>.
     * @param assigned_only If <i>true</i> only those languages currently having indexes built for them are checked, otherwise the known languages buffer is checked.
     * @return The <strong>Language</strong> that matches the given code, or <i>null</i> if no such language exists.
     */
    public Language getLanguage(String code, boolean assigned_only) {
	if(assigned_only) {
	    for(int i = 0; i < size(); i++) {
		Language pos = (Language)get(i);
		///ystem.err.println("Comparing " + pos.getCode() + " and " + code);
		if(pos.getCode().equals(code)) {
		    return pos;
		}
	    }
	}
	else {
	    if(known_languages.containsKey(code)) {
		return new Language((Language)known_languages.get(code));
	    }
	}
	return null;
    }
    /** Method to return a list of the known language codes.
     * @return An <strong>ArrayList</strong> containing a series of alphabetically sorted two letter codes.
     */
    public ArrayList getLanguageCodes() {
	ArrayList result = new ArrayList();
	Iterator key_iter = known_languages.keySet().iterator();
	while(key_iter.hasNext()) {
	    result.add(known_languages.get(key_iter.next()));
	}
	//for(Enumeration keys = known_languages.keys(); keys.hasMoreElements(); ) {
	//	result.add(known_languages.get(keys.nextElement()));
	//}
	//Collections.sort(result);
	return result;
    }
    /** Mark the current set of controls, if any, as obsolete and deallocate them. If further need of the controls will cause new controls to be created.
	* @see org.greenstone.gatherer.cdm.IndexManager.Control
	*/
    public void invalidateControls() {
	if(controls != null) {
	    controls.destroy();
	}
	controls = null;
    }

    /** Determine if the given language is the current default language.
	  * @param language The <strong>Language</strong> to test.
	  * @return <i>true</i> if the language is the default one, <i>false</i> otherwise.
	  */
    public boolean isDefaultLanguage(Language language) {
	return (language.equals(default_language));
    }
    /** This method attempts to parse a language command from the given string. If such a command is parsed, it is immediately added to the list of languages.
	  * @param command The <strong>String</strong> containing a possible language command.
	  * @return A <i>boolean</i> which is <i>true</i> if a command was parsed, <i>false</i> otherwise.
	  * @see org.greenstone.gatherer.cdm.Language
	  */
    public boolean parse(String command) {
	String command_lc = command.toLowerCase();
	if(command_lc.startsWith("languages")) {
	    StringTokenizer tokenizer = new StringTokenizer(command);
	    tokenizer.nextToken();
	    while(tokenizer.hasMoreTokens()) {
		String code = tokenizer.nextToken();
		if(known_languages.containsKey(code)) {
		    Language language = (Language)known_languages.get(code);
		    addElement(new Language(language));
		}
		else {
		    addElement(new Language(code, code, false));
		}
	    }
	    return true;
	}
	if(command_lc.startsWith("defaultlanguage")) {
	    StringTokenizer tokenizer = new StringTokenizer(command);
	    tokenizer.nextToken();
	    if(tokenizer.hasMoreTokens()) {
		String code = tokenizer.nextToken();
		Language language = getLanguage(code, true);
		if(language == null) {
		    language = new Language(code, (String)known_languages.get(code), true);
		    addElement(language);
		}
		else {
		    language.setDefault(true);
		}
		setDefault(language);
	    }
	    return true;
	}
	return false;
    }
    /** Method to cause the list appearance to update if the selection changes. */
    public void refreshAppearance() {
	fireContentsChanged(this, 0, size());
    }
    /** Method to remove a certain language.
	  * @param language The <strong>Language</strong> to remove.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  */
    public void removeLanguage(Language language) {
	removeElement(language);
	if(default_language != null && default_language.equals(language)) {
	    default_language = null;
	}
	gatherer.c_man.configurationChanged();
    }
    /** Method to set the default language.
	  * @param language The <strong>Language</strong> to use as a default, or <i>null</i> for no default.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  */
    public void setDefault(Language language) {
	// Unset existing.
	Language old = null;
	if(default_language != null) {
	    old = default_language;
	    default_language.setDefault(false);
	    default_language = null;
	}
	// Now set the new if its not null.
	if(language != null) {
	    default_language = language;
	    default_language.setDefault(true);
	}
	// Now cause the model to refresh any lists that are listening.
	int start = 0;
	int end = size() - 1;
	if(default_language != null && old != null) {
	    int index_default = indexOf(default_language);
	    int index_old = indexOf(old);
	    if(index_default < index_old) {
		start = index_default;
		end = index_old;
	    }
	    else {
		start = index_old;
		end = index_default;
	    }
	}
	else if(default_language != null) {
	    start = end = indexOf(default_language);
	}
	else {
	    start = end = indexOf(old);
	}
	fireContentsChanged(this, 0, size());
	gatherer.c_man.configurationChanged();
    }
    /** Method to translate this object into a block of commands as you you expect to find in the collection configuration file.
     * @return A <strong>String</string> containing a series of commands.
     */
    public String toString() {
	StringBuffer text = new StringBuffer();
	if(size() > 1) {
	    text.append("languages ");
	    for(int i = 0; i < size(); i++) {
		Language language = (Language) get(i);
		text.append(language.getCode());
		if(i < size() - 1) {
		    text.append(" ");
		}
		else {
		    text.append("\n");
		}
	    }
				// Only bother with default language if there is more than one language.
	    if(default_language != null) {
		text.append("defaultlanguage ");
		text.append(default_language.getCode());
		text.append("\n");
	    }
	    text.append("\n");
	}
	return text.toString();
    }
    /** Overloaded to call get with both a key and an empty argument array.
	  * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
	  * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
	  */
    private String get(String key) {
	return get(key, null);
    }
    /** Used to retrieve a property value from the Locale specific ResourceBundle, based upon the key and arguments supplied. If the key cannot be found or if some other part of the call fails a default (English) error message is returned. <BR>
	  * Here the get recieves a second argument which is an array of Strings used to populate argument fields, denoted {<I>n</I>}, within the value String returned. Note that argument numbers greater than or equal to 32 are automatically mapped to the formatting String named Farg<I>n</I>.
	  * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
	  * @param args A <strong>String[]</strong> used to populate argument fields within the complete String.
	  * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.Dictionary
	  */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "CDM.LanguageManager." + key;
	}
	return gatherer.dictionary.get(key, args);
    }
    /** This method loads a series of code->language mappings into known_languages, by reading from the 'languages.dat' file, which is essentially a verbatim copy of the ISO 639 Standard.
	 * @see org.greenstone.gatherer.cdm.Language
	  */
    private void loadLanguages() {
	try {
	    File in_file = new File("languages.dat");
	    FileReader in_reader = new FileReader(in_file);
	    BufferedReader in = new BufferedReader(in_reader);
	    String entry = null;
	    while((entry = in.readLine()) != null) {
		if(!entry.startsWith("#")) {
		    StringTokenizer tokenizer = new StringTokenizer(entry);
		    String name = tokenizer.nextToken();
		    String code = tokenizer.nextToken().toLowerCase();
		    Language language = new Language(code, name, false);
		    known_languages.put(code, language);
		}
	    }
	    in.close();
	}
	catch (Exception error) {
	    error.printStackTrace();
	}
    }

    /** This class represents the visual component of the Language Manager. */
    private class Control
	extends JPanel {
	/** The button to add a new language support. */
	private JButton add = null;
	/** The button to clear the current default language. */
	private JButton clear_default = null;
	/** The button to remove a supported language. */
	private JButton remove = null;
	/** The button to set the current language as the default one. */
	private JButton set_default = null;
	/** A combobox listing the available supported languages. */
	private JComboBox selector = null;
	/** A list of currently supported languages. */
	private JList list = null;
	/** A description of the language currently selected. */
	private JTextArea description = null;
	/** The text field showing the currently name of the default language. Non-editable. */
	private JTextField default_language_value = null;
	/** Constructor.
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.AddListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.ClearDefaultListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.ListListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.RemoveListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.SelectorListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.SetDefaultListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.Control.TranslateListener
	 */
	public Control() {
	    super();
	    // Creation.
	    JPanel center_panel = new JPanel();

	    JLabel title_label = new JLabel(get("Title"));
	    title_label.setHorizontalAlignment(JLabel.CENTER);
	    title_label.setOpaque(true);

	    list = new JList(model);
	    list.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	    list.setCellRenderer(new ListRenderer());

	    JPanel details_panel = new JPanel();

	    JPanel default_panel = new JPanel();

	    JLabel default_label = new JLabel(get("Default_Language"));
	    default_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
	    default_label.setPreferredSize(LABEL_SIZE);

	    if(default_language == null) {
		default_language_value = new JTextField();
	    }
	    else {
		default_language_value = new JTextField(default_language.toString());
	    }
	    default_language_value.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	    default_language_value.setEditable(false);

	    JPanel control_panel = new JPanel();

	    JLabel selector_label = new JLabel(get("Selector"));
	    selector_label.setPreferredSize(LABEL_SIZE);

	    selector = new JComboBox(getLanguageCodes().toArray());
	    selector.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));

	    JPanel button_panel = new JPanel();

	    add = new JButton(get("Add"));
	    add.setMnemonic(KeyEvent.VK_A);
	    if(selector.getSelectedItem() != null) {
		add.setEnabled(true);
	    }
	    else {
		add.setEnabled(false);
	    }

	    remove = new JButton(get("Remove"));
	    remove.setMnemonic(KeyEvent.VK_R);
	    remove.setEnabled(false);

	    clear_default = new JButton(get("Clear_Default"));
	    clear_default.setMnemonic(KeyEvent.VK_C);
	    clear_default.setEnabled(false);

	    set_default = new JButton(get("Set_Default"));
	    set_default.setMnemonic(KeyEvent.VK_S);
	    set_default.setEnabled(false);

	    // Set up and connect listeners.
	    add.addActionListener(new AddListener());
	    clear_default.addActionListener(new ClearDefaultListener());
	    remove.addActionListener(new RemoveListener());
	    selector.addActionListener(new SelectorListener());
	    set_default.addActionListener(new SetDefaultListener());
	    list.addListSelectionListener(new ListListener());
	    // Layout components.
	    default_panel.setBorder(BorderFactory.createRaisedBevelBorder());
	    default_panel.setLayout(new BorderLayout());
	    default_panel.add(default_label, BorderLayout.WEST);
	    default_panel.add(default_language_value, BorderLayout.CENTER);

	    control_panel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    control_panel.setLayout(new BorderLayout());
	    control_panel.add(selector_label, BorderLayout.WEST);
	    control_panel.add(selector, BorderLayout.CENTER);

	    details_panel.setBorder(BorderFactory.createEmptyBorder(5,0,5,0));
	    details_panel.setLayout(new BorderLayout());
	    details_panel.add(default_panel, BorderLayout.CENTER);
	    details_panel.add(control_panel, BorderLayout.SOUTH);

	    center_panel.setLayout(new BorderLayout());
	    center_panel.add(title_label, BorderLayout.NORTH);
	    center_panel.add(new JScrollPane(list), BorderLayout.CENTER);
	    center_panel.add(details_panel, BorderLayout.SOUTH);

	    button_panel.setLayout(new GridLayout(2,2,5,5));
	    button_panel.add(add);
	    button_panel.add(remove);
	    button_panel.add(clear_default);
	    button_panel.add(set_default);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(center_panel, BorderLayout.CENTER);
	    add(button_panel, BorderLayout.SOUTH);
	}
	/** Destructor.
			*/
	public void destroy() {
	}
	/** Listens for actions apon the 'add' button in the LanguageManager controls, and if detected calls the add method of the manager with a newly created language. */
	private class AddListener
	    implements ActionListener {
	    /** Add a new language support.
	     * @param event an ActionEvent
	     * @see org.greenstone.gatherer.cdm.Language
	     */
	    public void actionPerformed(ActionEvent event) {
		Language language = (Language)selector.getSelectedItem();
		if(language != null) {
		    addLanguage(new Language(language));
		}
	    }
	}
	/** Listens for actions apon the 'clear default' button in the LanguageManager controls, and if detected calls the setDefault method of the manager with <i>null</i>. */
	private class ClearDefaultListener
	    implements ActionListener {
	    /** Clear the default index.
	     * @param event An <strong>ActionEvent</strong>.
	     */
	    public void actionPerformed(ActionEvent event) {
		setDefault(null);
		clear_default.setEnabled(false);
		default_language_value.setText("");
	    }
	}
	/** Listens for actions apon the 'remove' button in the LanguageManager controls, and if detected calls the remove method of the manager with the language selected for removal. */
	private class RemoveListener
	    implements ActionListener {
	    /** Remove the currently selected language, if any.
	     * @param event An <strong>ActionEvent</strong>.
	     * @see org.greenstone.gatherer.cdm.Language
	     */
	    public void actionPerformed(ActionEvent event) {
		if(!list.isSelectionEmpty()) {
		    removeLanguage((Language)list.getSelectedValue());
		    if(default_language == null) {
			clear_default.setEnabled(false);
			default_language_value.setText("");
		    }
		}
	    }
	}
	/** Listens for selections wihtin the combobox on the LanguageManager controls, and if a change is detected enables, or disables, controls appropriately. */
	private class SelectorListener
	    implements ActionListener {
	    /** Enable or disable controls depeding on selection.
	     * @param event An <strong>ActionEvent</strong>.
	     */
	    public void actionPerformed(ActionEvent event) {
		if(selector.getSelectedItem() != null) {
		    add.setEnabled(true);
		    //description.setText((String)known_languages.get(code));
		}
		else {
		    add.setEnabled(false);
		    //description.setText("");
		}
	    }
	}
	/** Listens for actions apon the 'set default' button in the LanguageManager controls, and if detected calls the <i>setDefault()</i> method of the manager with the language selected for default. */
	private class SetDefaultListener
	    implements ActionListener {
				/** Set the default index to the one currently selected, if any.
				 * @param event An <strong>ActionEvent</strong>.
				 * @see org.greenstone.gatherer.cdm.Language
				 */
	    public void actionPerformed(ActionEvent event) {
		if(!list.isSelectionEmpty()) {
		    setDefault((Language)list.getSelectedValue());
		    clear_default.setEnabled(true);
		    default_language_value.setText(default_language.toString());
		}
	    }
	}
	/** Listens for selections within the list on the LanguageManager controls, and if a change is detected enables, or disables, controls appropriately. */
	private class ListListener
	    implements ListSelectionListener {
	    /** Enable or disable controls depending on the current list selection.
	     * @param event A <strong>ListSelectionEvent</strong>.
	     */
	    public void valueChanged(ListSelectionEvent event) {
		if(list.isSelectionEmpty()) {
		    remove.setEnabled(false);
		    set_default.setEnabled(false);
		}
		else {
		    remove.setEnabled(true);
		    set_default.setEnabled(true);
		}
		refreshAppearance();
	    }
	}
	/** Our list cel renderer which renders the default cell just a little different. */
	private class ListRenderer
	    extends DefaultListCellRenderer {
	    /** Method to produce the component used to display an entry in the list.
	     * @param list The <strong>JList</strong> the component will be placed in.
	     * @param value A value to be displayed for this component, as a <strong>Object</strong>.
	     * @param index The position in the list it will occupy as an <i>int</i>.
	     * @param isSelected Whether the user has currently selected this component, as a <i>boolean</i>.
	     * @param cellHasFocus Whether the component currently has focus, again as a <i>boolean</i>.
	     * @return A <strong>Component</strong> to be used as the entry in the list.
	     * @see org.greenstone.gatherer.cdm.Language
	     */
	    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		Component component = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		Language language = (Language) value;
		if(language.isDefault() && !isSelected) {
		    component.setBackground(Gatherer.config.getColor("coloring.workspace_selection_background", false));
		}
		if(component instanceof JLabel) {
		    ((JLabel)component).setOpaque(true);
		}
		return component;
	    }
	}
    }
}







